/*
 * file: Debug.js
 *
 * @BEGINLICENSE
 * Copyright 2010 Brook Novak  (email : brooknovak@seaweed-editor.com) 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 * @ENDLICENSE
 */
/**
 * @namespace The debug namespace will be removed in release builds.
 */
debug = {};

/**
 * Asserts a condition. If a condition fails a alert box shows and an exception is thrown.
 * 
 * @param {Boolean} cond A condition
 * 
 * @param {String} msg An optional message
 * 
 * @throws {Error} if a condition fails
 */
debug.assert = function(cond, msg) {

    try {
        if (!cond) throw new Error("Assertion failed" + (msg ? ": " + msg : ""));
    } catch (e) {
		
		var fullMsg = e.message + (e.stack ? "\nstack:\n" + e.stack : "");
		
		alert(fullMsg);

		throw e;
    }
}

debug.close = function(){
    
    debug.stop = true;
    
    if (typeof _debugConsole != "undefined" && _debugConsole) {
        
        _debugConsole.parentNode.removeChild(_debugConsole);
        
        _debugConsole = null;
        
    }
}

debug.stop = false;

/**
 * Prints a message to a debug console.
 * 
 * @param {String} msg A message to print.
 */
debug.print = function(msg){
    
    if (debug.stop)
        return;

    if (window.console && console.log) {
        console.log(msg);
    } else if (window.opera && window.opera.postError){
        window.opera.postError(msg);
    } else {
    
        // Setup debug console
        if (typeof _debugConsole == "undefined" || !_debugConsole) {

            _debugConsole = document.createElement("div");
            _debugConsole.style.backgroundColor = "#444444";
            _debugConsole.style.position = "fixed";
            _debugConsole.style.border = "2px solid #000000";
            _debugConsole.style.width = "320px";
            _debugConsole.style.height = "174px";
            _debugConsole.style.zIndex = "9999";
            _debugConsole.style.top = "0";
            _debugConsole.style.left = "0";
            
            _debugConsole.innerHTML = '<div style="width:320px; color:white; font-weight:bold; font-family:arial,sans; text-align:center; font-size:14px;"><div style="float:right;border: 1px dashed black; color:black; background-color:#66D390; padding:4px; cursor:pointer;" onclick="debug.close();">close</div>~Seaweed~ Debug Console</div>';
            
            _debugOutput = document.createElement("div");
            _debugOutput.style.backgroundColor = "white";
            _debugOutput.style.overflow = "scroll";
            _debugOutput.style.width = _debugConsole.style.width;
            _debugOutput.style.height = "140px";
            
            _debugConsole.appendChild(_debugOutput);
            
        }
        
        if (document.body && _debugConsole && _debugConsole.parentNode != document.body)
            document.body.appendChild(_debugConsole);
                
        _debugOutput.innerHTML += msg.replace(/\n/g,"<br/>");
        var st = _debugOutput.scrollHeight - _debugOutput.clientHeight;
        if (st < 0) st = 0;
        _debugOutput.scrollTop = st;

    }
}

/**
 * Prints a message to a debug console with a new line
 * 
 * @param {String} msg A message to print.
 */
debug.println = function(msg){
    debug.print(msg + "\n");
};

/**
 * Prints out a TODO message and it's stack location
 * @param {String} msg An optional todo message
 */
debug.todo = function(msg) {
    try {
       throw new Error();
    } catch (e) {
        var fullMsg = "TODO: " + (msg ? msg : "") + (e.stack ? "\nAt" + e.stack : + "");
        debug.println(fullMsg);
    }
};


