/*
 * file: Platform.js
 *
 * @BEGINLICENSE
 * Copyright 2010 Brook Novak  (email : brooknovak@seaweed-editor.com) 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 * @ENDLICENSE
 */
bootstrap.provides("Platform");

$enqueueInit("Platform", function() {
    
    // Detect text direction locale - relies on DOM being ready for manipulation
    
    var container = $createElement("p");
    container.style.margin = "0 0 0 0";
    container.style.padding = "0 0 0 0";
    // container.style.textAlign = "start"; // If CSS 3+ 
    container.style.textAlign = ""; // Explicitly override text align that might be assigned via style sheets
    
    var span = $createElement("span");
    span.innerHTML = "X";
    
    container.appendChild(span);
    docBody.appendChild(container);
    
    // LTR if text position is nearer left of container, RTL if text position is nearer right of container
    _localeDirection = span.offsetLeft < (container.offsetWidth - (span.offsetLeft + span.offsetWidth)) ?
        "ltr" : 
        "rtl";
        
    debug.println("LOCALE-DIRECTION: " + _localeDirection + "\n");
        
    // Tidy up
    docBody.removeChild(container);

});
  
/**
 * The clients operating system. Never null, but can be de.Platform.UNKNOWN.
 * @type Number
 */
var _os,

    /**
     * The clients browser. -1 if unknown.
     * @type Number
     */
    _browser,

    /**
     * The browser version as a float. Can be -1 if could not determine the version.
     * @type Number
     */
	_browserVersion,

    /**
     * The clients operating system. -1 if unknown.
     * @type Number
     */
	_engine,
    
    /**
     * The layout/rendering engine.  -1 if unavailable.
     * @type Number
     */
    _engineVersion,
    
    /**
     * An enumeration for browser/engine/os types.
     * In the release version these are replaced with actual values.
     */ 
    _Platform = {
    	
    	/**
    	 * Read Only
    	 * @final
    	 * @type Number
    	 */
    	UNKNOWN : -1, // @REPLACE _Platform.UNKNOWN -1
    	
    	/* BROWSER CONSTANTS */
       
    	/**
    	 * Read Only: A browser constant
    	 * @final
    	 * @type Number
    	 */
    	FIREFOX : 1, // @REPLACE _Platform.FIREFOX 1
    	
    	/**
    	 * Read Only: A browser constant
    	 * @final
    	 * @type Number
    	 */
    	OPERA : 2, // @REPLACE _Platform.OPERA 2
    	
    	/**
    	 * Read Only: A browser constant
    	 * @final
    	 * @type Number
    	 */
    	IE : 3, // @REPLACE _Platform.IE 3
    	
    	/**
    	 * Read Only: A browser constant
    	 * @final
    	 * @type Number
    	 */
    	CHROME : 4, // @REPLACE _Platform.CHROME 4
    	
    	/**
    	 * Read Only: A browser constant
    	 * @final
    	 * @type Number
    	 */
    	SAFARI : 6, // @REPLACE _Platform.SAFARI 6
    	//ICAB : 101, // Used as engine contant aswell
    	
    	/**
    	 * Read Only: A browser constant
    	 * @final
    	 * @type Number
    	 */
    	KONQUEROR : 8, // @REPLACE _Platform.KONQUEROR 8
    	
    	/**
    	 * Read Only: A browser constant
    	 * @final
    	 * @type Number
    	 */
    	NETSCAPE : 9, // @REPLACE _Platform.NETSCAPE 9
    
    	
    	// OS CONSTANTS
    	/**
    	 * Read Only: An OS constant
    	 * @final
    	 * @type Number
    	 */
    	WINDOWS : 1, // @REPLACE _Platform.WINDOWS 1
    	
    	/**
    	 * Read Only: An OS constant
    	 * @final
    	 * @type Number
    	 */
    	MAC : 2, // @REPLACE _Platform.MAC 2
    	
    	/**
    	 * Read Only: An OS constant
    	 * @final
    	 * @type Number
    	 */
    	LINUX : 3, // @REPLACE _Platform.LINUX 3

    	
    	/* LAYOUT/RENDERING ENGINE CONSTANTS */
    	
    	/**
    	 * Read Only: A rendering engine constant
    	 * @final
    	 * @type Number
    	 */
    	GECKO : 1,  // @REPLACE _Platform.GECKO 1
    	
    	/**
    	 * Read Only: A rendering engine constant
    	 * @final
    	 * @type Number
    	 */
    	TRIDENT : 2,  // @REPLACE _Platform.TRIDENT 2
    	
    	/**
    	 * Read Only: A rendering engine constant
    	 * @final
    	 * @type Number
    	 */
    	WEBKIT : 3,  // @REPLACE _Platform.WEBKIT 3
    	
    	/**
    	 * Read Only: A rendering engine constant
    	 * @final
    	 * @type Number
    	 */
    	KHTML : 4,  // @REPLACE _Platform.KHTML 4
    	
    	/**
    	 * Read Only: A rendering engine constant
    	 * @final
    	 * @type Number
    	 */
    	PRESTO : 5  // @REPLACE _Platform.PRESTO 5
    
    };


// Perform platform detection
(function(){

    // References:
    // - http://unixpapa.com/js/gecko.html
    //  - http://www.quirksmode.org/js/detect.html
    
    var dataBrowser = [{
        string: navigator.userAgent,
        subString: "Chrome",
        id: _Platform.CHROME,
        versionSearch: "Chrome"
    }, {
        string: navigator.vendor,
        subString: "Apple",
        id: _Platform.SAFARI,
        versionSearch: "Version"
    }, {
        prop: window.opera,
        id: _Platform.OPERA,
        versionSearch: "Opera"
    }, {
        string: navigator.vendor,
        subString: "KDE",
        id: _Platform.KONQUEROR,
        versionSearch: "Konqueror"
    }, {
        string: navigator.userAgent,
        subString: "Firefox",
        id: _Platform.FIREFOX,
        versionSearch: "Firefox"
    }, {
        // for newer Netscapes (6+)
        string: navigator.userAgent,
        subString: "Netscape",
        id: _Platform.NETSCAPE,
        versionSearch: "Netscape"
    }, {
        string: navigator.userAgent,
        subString: "MSIE",
        id: _Platform.IE,
        versionSearch: "MSIE"
    }, {
        // for older Netscapes (4-)
        string: navigator.userAgent,
        subString: "Mozilla",
        id: _Platform.NETSCAPE,
        versionSearch: "Mozilla"
    }];
    
    var dataOS = [{
        string: navigator.platform,
        subString: "Win",
        id: _Platform.WINDOWS
    }, {
        string: navigator.platform,
        subString: "Mac",
        id: _Platform.MAC
    }, {
        string: navigator.platform,
        subString: "Linux",
        id: _Platform.LINUX
    }, ];
    
    var dataEngine = [{
        string: navigator.userAgent,
        subString: "MSIE",
        id: _Platform.TRIDENT,
        versionSearch: "MSIE" // The trident versions are same as browser versions
    }, {
        // It is important that this is above KHTML - since webkit is forked from KHTML  (which is still in webkits useragent/nav strings)
        string: navigator.userAgent,
        subString: "WebKit",
        id: _Platform.WEBKIT,
        versionSearch: "WebKit"
    }, {
        // It is important to have this above gecko data. since the user agent
        // can contain gecko
        string: navigator.userAgent,
        subString: "KHTML",
        id: _Platform.KHTML,
        versionSearch: "KHTML"
    }, {
        string: navigator.userAgent,
        subString: "Gecko",
        id: _Platform.GECKO,
        versionSearch: "rv"
    }, {
        prop: window.opera,
        id: _Platform.PRESTO,
        versionSearch: "Presto"
    }, ];
    
    function findMatchingPlatform(data){
    
        for (var i in data) {
        
            var dataString = data[i].string,
                dataProp = data[i].prop;
            
            if (dataString) {
                if (dataString.indexOf(data[i].subString) != -1)                     
                    return data[i];
            } else if (dataProp)                 
                return data[i];
        }
        
        return null;
    }
    
    function extractVersion(dataString, versionSearchString){
        if (!versionSearchString)             
            return null;
        var index = dataString.indexOf(versionSearchString);
        if (index == -1)             
            return null;
        return parseFloat(dataString.substring(index + versionSearchString.length + 1)); // Add one for the "/" between the identifier and the version
    }
    
    debug.println("Inferring platform...");
    
    // Get the OS
    _os = findMatchingPlatform(dataOS);
    
    debug.println("OS: " + (_os ? _os.subString : "UNKNOWN"));
    
    // Set as the enum...
    _os = _os ? _os.id || _Platform.UNKNOWN : _Platform.UNKNOWN;
    
    // Get the browser
    _browser = findMatchingPlatform(dataBrowser);
    
    debug.println("BROWSER: " + (_browser ? _browser.versionSearch : "UNKNOWN"));
    
    _browserVersion = _Platform.UNKNOWN;
    
    if (_browser) {
    
        // Extract the version
        _browserVersion = extractVersion(navigator.userAgent, _browser.versionSearch) ||
        extractVersion(navigator.appVersion, _browser.versionSearch);
        
        // Set browser as the enum	
        _browser = _browser.id;
        
    } else 
        _browser = _Platform.UNKNOWN;
    
    debug.println("BROWSER-VERSION: " + _browserVersion);
    
    
    // Get the layout engine
    _engine = findMatchingPlatform(dataEngine);
    _engineVersion = _Platform.UNKNOWN;
    
    debug.println("ENGINE: " + (_engine ? (_engine.subString ? _engine.subString : _engine.versionSearch) : "UNKNOWN"));
    
    
    if (_engine) {
        _engineVersion = extractVersion(navigator.userAgent, _engine.versionSearch);
        _engine = _engine.id;
    } else 
        _engine = _Platform.UNKNOWN;
    
    debug.println("ENGINE-VERSION: " + _engineVersion + "\n");

})();

$extend(de, {
    
    /**
     * @memberOf de
     * Exposes internal platform enumaration
     * @see _Platform
     */
    Platform : _Platform,
    
    /**
     * @memberOf de
     * Exposes internal field
     * @see _os
     */
    os : _os,
    
    /**
     * @memberOf de
     * Exposes internal field
     * @see _browser
     */
    browser : _browser,
    
    /**
     * @memberOf de
     * Exposes internal field
     * @see _browserVersion
     */
    browserVersion : _browserVersion,
    
    /**
     * @memberOf de
     * Exposes internal field
     * @see _engine
     */
    engine : _engine,
    
    /**
     * @memberOf de
     * Exposes internal field
     * @see _engineVersion
     */
    engineVersion : _engineVersion,
    
    /**
     * @memberOf de
     * The Local text direction. Either "ltr" for Left to right or "rtl" for right to left.
     * @type String
     */
    localDirection : null /* Detected once API initialized */
      
});
    
 
