/*
 * file: Viewport.js
 *
 * @BEGINLICENSE
 * Copyright 2010 Brook Novak  (email : brooknovak@seaweed-editor.com) 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 * @ENDLICENSE
 */

bootstrap.provides("viewport");

// Internal declarations
var _getViewportSize, _getScrollbarThickness, _getBodyOffset, _getDocumentScrollPos;

(function(){

    var 
    
    recalcViewportPending = 1,
    cachedVPWidth,
    cachedVPHeight; 
    
    $enqueueInit("viewport", function(){
       
        // On resize invalidate scrollbars and body offset measurements
        _addHandler(window, "resize", function(){
            recalcViewportPending = 1;
        });
        
    });
    
    /**
     * @param {Boolean} forceReCalc True to force a relcalculation of the viewport dimension.
     *                              Otherwised it may return a cached version. Only need to set to true
     *                              if calling within a onresize event (since cached values are re-evaluated
     *                              on resize events).
     *
     * @return {Object} The size of the documents viewport - excluding scrollbars. {width, height}
     */
    de.getViewPortSize = _getViewPortSize = function(forceRecalc){
        
        if (recalcViewportPending || forceRecalc) 
            reCalcViewport();
        
        return {
            width: cachedVPWidth,
            height: cachedVPHeight
        };
        
    };
    
    /**
     * Recalculates the viewport dimensions
     */
    function reCalcViewport() {
        
        if (_browser == _Platform.IE && _browserVersion < 7) {
            // IE 6 and below does not support fixed positioned elements and has troubles with
            // 100% heights .. can just query doc element client area
            cachedVPWidth = document.documentElement.clientWidth;
            cachedVPHeight = document.documentElement.clientHeight; // TODO: For all IE's? 
            
        } else {
        
            var measureDiv = $createElement("div");
            
            _setFullStyle(measureDiv, "position:fixed;top:0;left:0;width:100%;height:100%;border-style:none;margin:0");
            
            docBody.appendChild(measureDiv);
            
            cachedVPWidth = measureDiv.offsetWidth;
            cachedVPHeight = measureDiv.offsetHeight;
            
            docBody.removeChild(measureDiv);
        }
        
        recalcViewportPending = 0;
        
    }
    
})();


/**
 * @return {Object} The current scroll position of the document {top, left}.
 */
de.getDocumentScrollPos = _getDocumentScrollPos = function(){

    var left = 0, top = 0;
    
    // DOM Compliant?
    if (docBody.scrollLeft || docBody.scrollTop) {
    
        left = docBody.scrollLeft;
        top = docBody.scrollTop;
    
    } else if (window.pageYOffset || window.pageXOffset) {
    
        left = window.pageXOffset;
        top = window.pageYOffset;
    
    } else if (document.documentElement.scrollLeft || document.documentElement.scrollTop) {
        
        left = document.documentElement.scrollLeft;
        top = document.documentElement.scrollTop;
    
    }
    
    return {
        top: top,
        left: left
    };
}

/**
 * Note: this does not include the browser toolbars/status bars etc.. just the document viewing area...
 * @return {Object} the size of the documents window - i.e. including scrollbars. {width, height}
 */
function _getWindowSize(){

    var width = 0, height = 0;
    
    if (window.innerWidth || window.innerHeight) { // Most Browsers
    
        width = window.innerWidth;
        height = window.innerHeight;
        
    } else if (document.documentElement.offsetWidth || document.documentElement.offsetWidth) { // IE
    
        width = document.documentElement.offsetWidth;
        height = document.documentElement.offsetHeight;
        
    } else if (docBody.offsetWidth || docBody.offsetWidth) { 

        width = docBody.offsetWidth;
        height = docBody.offsetWidth;
        
    }
    
    return {
        width: width,
        height: height
    };
};


