/*
 * file: ChangeContainerAction.js
 *
 * @BEGINLICENSE
 * Copyright 2010 Brook Novak  (email : brooknovak@seaweed-editor.com) 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 * @ENDLICENSE
 */

// @DEPENDS: UndoMan
bootstrap.provides("actions.ChangeContainerAction");

(function() {
    
    var changeBlockMap = $createLookupMap("p,pre,h1,h2,h3,h4,h5,h6,address");
    
    _registerAction("ChangeContainer", {
        
        /**
         * An undoable action. Changes all block level elements in a given range - creates new containers where needed.
         * 
         * @author Brook Novak
         * 
         * @param {String} containerTag (Optional) The container nodename to encapsulate the range. 
         * 
         * @param {Node} startNode      (Optional) The starting dom node of the range to align.
         *                              If not provided then the current selection will be used.
         *                              If provieded must also provide endNode
         *
         * @param {Node} endNode        (Optional) The ending dom node of the range to align. Can be the same as start node
         *                              If not provided then the current selection will be used.
         * 
         */
        exec : function(containerTag, startNode, endNode) {
			
			if (!containerTag)
				containerTag = "p";
			containerTag = containerTag.toLowerCase();
				
			debug.assert(changeBlockMap[containerTag], "Cannot change container type to \"" + changeBlockMap + "\"");
			
            // Auto-set range if not provided.
            if (!startNode) {
                
                if (!this.selBefore)
                    return; // Nothing to select
                    
                if (this.selBefore.endNode) {
                    startNode = this.selBeforeOrdered.startNode;
                    endNode = this.selBeforeOrdered.endNode;
                } else 
                    startNode = endNode = this.selBefore.startNode;
                
            }
            
            debug.assert(endNode, "Supplied start node but not the end node");
            
            
            var containers = _getNormalizedContainerRange(startNode, endNode);
			
			for (var i in containers) {
				
				var container = containers[i];
				
				// Can this block level container be changed to a certain type?
				if (_nodeName(container) != containerTag && changeBlockMap[_nodeName(container)]) {
					
					var migrants = [], 
						migrant,
						parent = container.parentNode,
						index = _indexInParent(container);
					
					// Remove container contents
					while(migrant = container.firstChild) {
						migrants.push(migrant);
						_execOp(_Operation.REMOVE_NODE, migrant);
					}
					
					// Remove container
					_execOp(_Operation.REMOVE_NODE, container);
					
					// Insert new container
					var newContainer = $createElement(containerTag);
					_execOp(_Operation.INSERT_NODE, newContainer, parent, index);
					
					// Migrate original container elements
					for (var j in migrants) {
						_execOp(_Operation.INSERT_NODE, migrants[j], newContainer);
					}
					
				}
				
			}
            
            this.selAfter = this.selBefore;

        }
		
    });
    
})();
