/*
 * file: DemoteItemAction.js
 *
 * @BEGINLICENSE
 * Copyright 2010 Brook Novak  (email : brooknovak@seaweed-editor.com) 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 * @ENDLICENSE
 */

// @DEPENDS: UndoMan
bootstrap.provides("actions.DemoteItemAction");

_registerAction("DemoteItem", {
    
    exec : function(startNode, endNode) {
        
        var t = this;

        // Auto-set range if not provided.
        if (!startNode) {
            
            if (!t.selBefore)
                return; // Nothing to promote
                
            if (t.selBefore.endNode) {
                startNode = t.selBeforeOrdered.startNode;
                endNode = t.selBeforeOrdered.endNode;
            } else 
                startNode = endNode = t.selBefore.startNode;
            
        }
        
        debug.assert(endNode, "Supplied start node but not the end node");

        var ca = _findAncestor(_getCommonAncestor(startNode, endNode, 1), docBody, function(testNode) {
            var nn = _nodeName(testNode);
            return nn == "li" || nn == "ol" || nn == "ul";
        }, 1);

        // Found a list item / list container in the given range?
        if (ca) {
            
            // Get all list items in the given range
            var listItems = [];
            if (_nodeName(ca) == "li") 
                listItems.push(ca);
            else {
                _visitAllNodes(ca, startNode, true, function(domNode){
                    if (_nodeName(domNode) == "li") listItems.push(domNode);
                    return domNode != endNode;
                });
                // The traversal above will skip the start node's li
                var li = _findAncestor(startNode, ca, function(testNode) {return _nodeName(testNode) == "li";}, 1);
                if (li)
                    listItems.push(li);
            }
            
            // TODO: REFACTOR: ALL ABOVE IS IDENTICLE TO PROMOTE ACTION
            
            // Demote LI: Top => down
            for (var i in listItems) {
                demoteLI(listItems[i]);
            }
            
        }
        
        t.selAfter = t.selBefore;

        function demoteLI(li){
        
            // Can this list item be demoted? Its list container must be within a list container
            var targetLICon = li.parentNode.parentNode;
            if (_nodeName(targetLICon) == "ul" || _nodeName(targetLICon) == "ol") {
            
                // Is there a following list item at the same level
                var nextLI = li.nextSibling, prevLI = li.previousSibling; // lis, uls or ols,
                while (prevLI && prevLI.nodeType == Node.TEXT_NODE) {
                    prevLI = prevLI.previousSibling;
                }
                while (nextLI && nextLI.nodeType == Node.TEXT_NODE) {
                    nextLI = nextLI.nextSibling;
                }
                
                var insertIndex;
                
                if (nextLI && prevLI) { // List item to demote surrounded by others
                
                    // Split the contain in two
                    var lowerSplit = li.parentNode.cloneNode(false);
                    _execOp(_Operation.INSERT_NODE, lowerSplit, targetLICon, _indexInParent(li.parentNode) + 1);
                    
                    // Move following list items into split
                    while (nextLI) {
                        var migrant = nextLI;
                        nextLI = nextLI.nextSibling;
                        _execOp(_Operation.REMOVE_NODE, migrant);
                        _execOp(_Operation.INSERT_NODE, migrant, lowerSplit);
                    }
                    
                    insertIndex = _indexInParent(lowerSplit);
                    
                } else if (nextLI) {  // List item to demote contains another afterwards
                
                    insertIndex = _indexInParent(li.parentNode);
                    
                } else if (prevLI) {  // List item to demote contains another before
                
                    insertIndex = _indexInParent(li.parentNode) + 1;
                    
                } else { // List item to demote on its own
                
                    insertIndex = _indexInParent(li.parentNode);
                    
                    // Remove the container
                    _execOp(_Operation.REMOVE_NODE, li.parentNode);
                    
                }
                
                // Remove the li
                _execOp(_Operation.REMOVE_NODE, li);
                
                // Add it to the parent container
                _execOp(_Operation.INSERT_NODE, li, targetLICon, insertIndex);
                
            }
            
        }
        
    }
});

