/*
 * file: InsertTextAction.js
 *
 * @BEGINLICENSE
 * Copyright 2010 Brook Novak  (email : brooknovak@seaweed-editor.com) 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 * @ENDLICENSE
 */

// @DEPENDS: UndoMan
bootstrap.provides("actions.InsertTextAction");

_registerAction("InsertText", {
    
    exec : function(domNode, text, index) {
        
    	debug.assert(index >= 0);
        
    	debug.assert(
    		(domNode.nodeType == Node.TEXT_NODE && index <= domNode.nodeValue.length) || 
    		(domNode.nodeType == Node.ELEMENT_NODE && index <= 1)
    	);
        
        text = _parseHTMLString(_escapeTextToHTML(text)); // Escape into HTML and convert back into text with correct encoding

		var phRoot; 
        
		// If the target node is a placeholder, then the placeholder should be replaced with the new text
        if (de.doc.isESPlaceHolder(domNode, false) || de.doc.isMNPlaceHolder(domNode, false)) {
            // Get the placeholder root node
            phRoot = domNode;
            while (!(de.doc.isESPlaceHolder(phRoot, true) || de.doc.isMNPlaceHolder(phRoot, true))) {
                phRoot = phRoot.parentNode;
            }
            domNode = phRoot;
        }
		
		if (domNode.nodeType == Node.TEXT_NODE) {
            
            debug.assert(!phRoot);
			
            // Insert the text
            _execOp(_Operation.INSERT_TEXT, domNode, text, index);
			
            // Set the cursor to after the inserted text
            if (this.flags & de.UndoMan.ExecFlag.UPDATE_SELECTION)
                this.selAfter = {startNode : domNode, startIndex: index + text.length};
			
		} else {

			// Create a new text node and add it to the document
			var textNode = document.createTextNode(text);

            _execOp(_Operation.INSERT_NODE, textNode, domNode.parentNode, _indexInParent(domNode) + (index == 0 ? 0 : 1));
	
            // Remove the placeholder from the document if there was any
            if (phRoot) 
                _execOp(_Operation.REMOVE_NODE, phRoot);

            if (this.flags & de.UndoMan.ExecFlag.UPDATE_SELECTION)
                this.selAfter = {startNode : textNode, startIndex: index + _nodeLength(textNode)};
			
		}
        
        // Normalize any inserted non breaking spaces
        if (text.indexOf(_NBSP) != -1) 
            _normalizeNBSP(domNode.parentNode);

    }
});

