/*
 * file: PromoteItemAction.js
 *
 * @BEGINLICENSE
 * Copyright 2010 Brook Novak  (email : brooknovak@seaweed-editor.com) 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 * @ENDLICENSE
 */

// @DEPENDS: UndoMan
bootstrap.provides("actions.PromoteItemAction");

_registerAction("PromoteItem", {
    
    exec : function(startNode, endNode) {
        
        var t = this;

        // Auto-set range if not provided.
        if (!startNode) {
            
            if (!t.selBefore)
                return; // Nothing to promote
                
            if (t.selBefore.endNode) {
                startNode = t.selBeforeOrdered.startNode;
                endNode = t.selBeforeOrdered.endNode;
            } else 
                startNode = endNode = t.selBefore.startNode;
            
        }
        
        debug.assert(endNode, "Supplied start node but not the end node");

        var ca = _findAncestor(_getCommonAncestor(startNode, endNode, 1), docBody, function(testNode) {
            var nn = _nodeName(testNode);
            return nn == "li" || nn == "ol" || nn == "ul";
        }, 1);

        // Found a list item / list container in the given range?
        if (ca) {
            
            // Get all list items in the given range
            var listItems = [];
            if (_nodeName(ca) == "li") 
                listItems.push(ca);
            else {
                _visitAllNodes(ca, startNode, true, function(domNode){
                    if (_nodeName(domNode) == "li") listItems.push(domNode);
                    return domNode != endNode;
                });
                // The traversal above will skip the start node's li
                var li = _findAncestor(startNode, ca, function(testNode) {return _nodeName(testNode) == "li";}, 1);
                if (li)
                    listItems.push(li);
            }
            
            // Promote LI: Top => down
            for (var i in listItems) {
                promoteLI(listItems[i]);
            }
            
        }
        
        t.selAfter = t.selBefore;

        function promoteLI(li) {
            
            // Can this list item be promoted? It must have a list item at the same level preceeding it
            var prevLICon = li.previousSibling, nextLICon = li.nextSibling;
            while (prevLICon && prevLICon.nodeType == Node.TEXT_NODE) {
                prevLICon = prevLICon.previousSibling;
            }
            while (nextLICon && nextLICon.nodeType == Node.TEXT_NODE) {
                nextLICon = nextLICon.nextSibling;
            }
            
            var subList;
            
            if (prevLICon && (_nodeName(prevLICon) == "ol" || _nodeName(prevLICon) == "ul")) // Add to end of exisiting sb-list within previous LI?
                subList = prevLICon;
            else {
                // Create a new LI->[OL/UL] and insert it before the LI to promote
                subList = li.parentNode.cloneNode(false);
                _execOp(_Operation.INSERT_NODE, subList, li.parentNode, _indexInParent(li));
            }
            
            // Move the LI into the sublist
            _execOp(_Operation.REMOVE_NODE, li);
            _execOp(_Operation.INSERT_NODE, li, subList);
            
            // Check if need to merge follow list items
            if (nextLICon && _nodeName(nextLICon) == _nodeName(subList)) {

                // If so then merge the list containers

                // Merge the lists
                _execOp(_Operation.REMOVE_NODE, nextLICon);
                while(nextLICon.firstChild) {
                    var migrant = nextLICon.firstChild;
                    _execOp(_Operation.REMOVE_NODE, migrant);
                    _execOp(_Operation.INSERT_NODE, migrant, subList);
                }

            }
        }
        
        
    }
});

