/*
 * file: TextAlignAction.js
 *
 * @BEGINLICENSE
 * Copyright 2010 Brook Novak  (email : brooknovak@seaweed-editor.com) 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 * @ENDLICENSE
 */

// @DEPENDS: UndoMan
bootstrap.provides("actions.TextAlignAction");

(function() {
    
    var excludeAlignBlocks = $createLookupMap("dt,dd,caption,colgroup,col,thead,tfoot,tbody,legend,optgroup,option,area,frame");
    
    _registerAction("TextAlign", {
        
        /**
         * An undoable alignment action. Sets the text alignment for all block levels in given range.
         * Creates new containers on the fly if needed.
         * 
         * @author Brook Novak
         * 
         * @param {String} alignment    The CSS text-alignment value. Either left, right, center or justify.
         * 
         * @param {Node} startNode      (Optional) The starting dom node of the range to align.
         *                              If not provided then the current selection will be used.
         *                              If provieded must also provide endNode
         *
         * @param {Node} endNode        (Optional) The ending dom node of the range to align. Can be the same as start node
         *                              If not provided then the current selection will be used.
         *
         */
        exec : function(alignment, startNode, endNode) {
            
            // Auto-set range if not provided.
            if (!startNode) {
                
                if (!this.selBefore)
                    return; // Nothing to select
                    
                if (this.selBefore.endNode) {
                    startNode = this.selBeforeOrdered.startNode;
                    endNode = this.selBeforeOrdered.endNode;
                } else 
                    startNode = endNode = this.selBefore.startNode;
                
            }
            
            debug.assert(endNode, "Supplied start node but not the end node");

            var containers;
            
            // First check for special case: If the ranges first block level common ancestor
            // is a list item, then set alignment for the list item rather than normalizing within a list item.
            var ca = _getCommonAncestor(startNode, endNode);
            for (var level = 0; level < 2; level++) {
                while (ca != docBody && !_isBlockLevel(ca)) {
                    ca = ca.parentNode;
                }
                
                if (ca == docBody)
                    break;
                    
                if (_nodeName(ca) == "li") {
                    containers = [ca];
                    break;
                }
                ca = ca.parentNode;
            }
            
            if (!containers) 
                // Normalize containers in range and get list of all the containers
                containers = _getNormalizedContainerRange(startNode, endNode);
            
            for (var i in containers) {
                // NOTES: CSS 2+ spec allows text-align style to be applied to all block level elements.
                _visitAllNodes(containers[i], containers[i], true, function(domNode) {
                   if (_isBlockLevel(domNode) && !excludeAlignBlocks[_nodeName(domNode)]) {
                       _execOp(_Operation.SET_CSS_STYLE, domNode, "textAlign",  alignment);
                   }
                });
            }
            
            this.selAfter = this.selBefore;
        }
    });
        
})();
